import time

from django.test import LiveServerTestCase
from selenium import webdriver

from core import models
from core.models import UserAccount
from functional_tests.util import login, create_browser, extract_hrefs_hashes
from util import factory_boys as fact


class UntestedParent:
    class FrontPageTestsTutorReviewer(LiveServerTestCase):
        browser: webdriver.Firefox = None
        username = None
        password = None
        role = None

        def setUp(self):
            fact.SubmissionFactory.create_batch(4)

        def _login(self):
            login(self.browser, self.live_server_url, self.username, self.password)

        def test_statistics_are_shown(self):
            self._login()
            statistics = self.browser.find_element_by_id('correction-statistics')
            title = statistics.find_element_by_class_name('title')
            self.assertEqual('Statistics', title.text)

        def test_available_tasks_are_shown(self):
            self._login()
            tasks = self.browser.find_element_by_name('subscription-list')
            title = tasks.find_element_by_class_name('v-toolbar__title')
            self.assertEqual('Tasks', title.text)
            time.sleep(8)
            subscription_links = extract_hrefs_hashes(
                tasks.find_elements_by_tag_name('a')
            )
            subscriptions = models.SubmissionSubscription.objects.filter(
                owner__username=self.username,
                deactivated=False,
                feedback_stage=models.SubmissionSubscription.FEEDBACK_CREATION
            ).exclude(query_type=models.SubmissionSubscription.RANDOM)
            for sub in subscriptions:
                self.assertIn(f'/subscription/{sub.pk}', subscription_links)


class FrontPageTestsTutor(UntestedParent.FrontPageTestsTutorReviewer):
    def setUp(self):
        super().setUp()
        self.browser = create_browser()
        self.username = 'tutor'
        self.password = 'p'
        self.role = UserAccount.TUTOR
        fact.UserAccountFactory(
            username=self.username,
            password=self.password
        )

    def tearDown(self):
        self.browser.quit()

    def test_side_bar_contains_correct_items(self):
        self._login()
        drawer = self.browser.find_element_by_class_name('v-navigation-drawer')
        links = extract_hrefs_hashes(drawer.find_elements_by_tag_name('a'))
        print(links)
        self.assertTrue(all(link in links for link in ['/home', '/feedback']))
        task_title = drawer.find_element_by_class_name('v-toolbar__title')
        self.assertEqual('Tasks', task_title.text)
        footer = drawer.find_element_by_class_name('sidebar-footer')
        feedback_link = footer.find_element_by_css_selector('a.feedback-link')
        self.assertEqual('Give us Feedback!', feedback_link.text)
        self.assertEqual('https://gitlab.gwdg.de/j.michal/grady/issues',
                         feedback_link.get_attribute('href'))


class FrontPageTestsReviewer(UntestedParent.FrontPageTestsTutorReviewer):
    def setUp(self):
        super().setUp()
        self.browser = create_browser()
        self.browser = create_browser()
        self.username = 'reviewer'
        self.password = 'p'
        self.role = UserAccount.REVIEWER
        fact.UserAccountFactory(
            username=self.username,
            password=self.password,
            role=self.role
        )

    def tearDown(self):
        self.browser.quit()

    def test_side_bar_contains_correct_items(self):
        self._login()
        drawer = self.browser.find_element_by_class_name('v-navigation-drawer')
        links = extract_hrefs_hashes(drawer.find_elements_by_tag_name('a'))
        self.assertTrue(all(link in links for link in
                            ['/home', '/feedback', '/student-overview', '/tutor-overview']))
        task_title = drawer.find_element_by_class_name('v-toolbar__title')
        self.assertEqual('Tasks', task_title.text)
        footer = drawer.find_element_by_class_name('sidebar-footer')
        feedback_link = footer.find_element_by_css_selector('a.feedback-link')
        self.assertEqual('Give us Feedback!', feedback_link.text)
        self.assertEqual('https://gitlab.gwdg.de/j.michal/grady/issues',
                         feedback_link.get_attribute('href'))
