import os
import time
from itertools import islice
from typing import Sequence

from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.firefox.options import Options
from selenium.webdriver.remote.webelement import WebElement


def create_browser():
    options = Options()
    options.headless = bool(os.environ.get('HEADLESS_TESTS', False))
    browser = webdriver.Firefox(options=options)
    browser.implicitly_wait(5)
    return browser


def login(browser, live_server_url, username, password='p'):
    browser.get(live_server_url)
    username_input = browser.find_element_by_xpath('//input[@aria-label="Username"]')
    username_input.send_keys(username)
    password_input = browser.find_element_by_xpath('//input[@aria-label="Password"]')
    password_input.send_keys(password)
    browser.find_element_by_xpath('//button[@type="submit"]').send_keys(Keys.ENTER)
    time.sleep(1)


def nth(iterable, n, default=None):
    "Returns the nth item or a default value"
    return next(islice(iterable, n, None), default)


def extract_hrefs_hashes(web_elements: Sequence[WebElement]):
    return [nth(el.get_attribute('href').split('#'), 1, '')
            for el in web_elements if el.get_attribute('href')]
